# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import core_pb2 as core_dot_grpc_dot_core__pb2


class BasketControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.AddItem = channel.unary_unary(
                '/darsman.core.BasketController/AddItem',
                request_serializer=core_dot_grpc_dot_core__pb2.AddItemRequest.SerializeToString,
                response_deserializer=core_dot_grpc_dot_core__pb2.Response.FromString,
                )
        self.ApplyDiscount = channel.unary_unary(
                '/darsman.core.BasketController/ApplyDiscount',
                request_serializer=core_dot_grpc_dot_core__pb2.BasketApplyDiscountRequest.SerializeToString,
                response_deserializer=core_dot_grpc_dot_core__pb2.Response.FromString,
                )
        self.AssignOwner = channel.unary_unary(
                '/darsman.core.BasketController/AssignOwner',
                request_serializer=core_dot_grpc_dot_core__pb2.BasketAssignOwnerRequest.SerializeToString,
                response_deserializer=core_dot_grpc_dot_core__pb2.Response.FromString,
                )
        self.Create = channel.unary_unary(
                '/darsman.core.BasketController/Create',
                request_serializer=core_dot_grpc_dot_core__pb2.BasketRequest.SerializeToString,
                response_deserializer=core_dot_grpc_dot_core__pb2.BasketResponse.FromString,
                )
        self.RemoveItem = channel.unary_unary(
                '/darsman.core.BasketController/RemoveItem',
                request_serializer=core_dot_grpc_dot_core__pb2.RemoveItemRequest.SerializeToString,
                response_deserializer=core_dot_grpc_dot_core__pb2.Response.FromString,
                )
        self.Retrieve = channel.unary_unary(
                '/darsman.core.BasketController/Retrieve',
                request_serializer=core_dot_grpc_dot_core__pb2.BasketRetrieveRequest.SerializeToString,
                response_deserializer=core_dot_grpc_dot_core__pb2.BasketResponse.FromString,
                )


class BasketControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def AddItem(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ApplyDiscount(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AssignOwner(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Create(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RemoveItem(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Retrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_BasketControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'AddItem': grpc.unary_unary_rpc_method_handler(
                    servicer.AddItem,
                    request_deserializer=core_dot_grpc_dot_core__pb2.AddItemRequest.FromString,
                    response_serializer=core_dot_grpc_dot_core__pb2.Response.SerializeToString,
            ),
            'ApplyDiscount': grpc.unary_unary_rpc_method_handler(
                    servicer.ApplyDiscount,
                    request_deserializer=core_dot_grpc_dot_core__pb2.BasketApplyDiscountRequest.FromString,
                    response_serializer=core_dot_grpc_dot_core__pb2.Response.SerializeToString,
            ),
            'AssignOwner': grpc.unary_unary_rpc_method_handler(
                    servicer.AssignOwner,
                    request_deserializer=core_dot_grpc_dot_core__pb2.BasketAssignOwnerRequest.FromString,
                    response_serializer=core_dot_grpc_dot_core__pb2.Response.SerializeToString,
            ),
            'Create': grpc.unary_unary_rpc_method_handler(
                    servicer.Create,
                    request_deserializer=core_dot_grpc_dot_core__pb2.BasketRequest.FromString,
                    response_serializer=core_dot_grpc_dot_core__pb2.BasketResponse.SerializeToString,
            ),
            'RemoveItem': grpc.unary_unary_rpc_method_handler(
                    servicer.RemoveItem,
                    request_deserializer=core_dot_grpc_dot_core__pb2.RemoveItemRequest.FromString,
                    response_serializer=core_dot_grpc_dot_core__pb2.Response.SerializeToString,
            ),
            'Retrieve': grpc.unary_unary_rpc_method_handler(
                    servicer.Retrieve,
                    request_deserializer=core_dot_grpc_dot_core__pb2.BasketRetrieveRequest.FromString,
                    response_serializer=core_dot_grpc_dot_core__pb2.BasketResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'darsman.core.BasketController', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class BasketController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def AddItem(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.core.BasketController/AddItem',
            core_dot_grpc_dot_core__pb2.AddItemRequest.SerializeToString,
            core_dot_grpc_dot_core__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ApplyDiscount(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.core.BasketController/ApplyDiscount',
            core_dot_grpc_dot_core__pb2.BasketApplyDiscountRequest.SerializeToString,
            core_dot_grpc_dot_core__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AssignOwner(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.core.BasketController/AssignOwner',
            core_dot_grpc_dot_core__pb2.BasketAssignOwnerRequest.SerializeToString,
            core_dot_grpc_dot_core__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Create(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.core.BasketController/Create',
            core_dot_grpc_dot_core__pb2.BasketRequest.SerializeToString,
            core_dot_grpc_dot_core__pb2.BasketResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RemoveItem(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.core.BasketController/RemoveItem',
            core_dot_grpc_dot_core__pb2.RemoveItemRequest.SerializeToString,
            core_dot_grpc_dot_core__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Retrieve(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.core.BasketController/Retrieve',
            core_dot_grpc_dot_core__pb2.BasketRetrieveRequest.SerializeToString,
            core_dot_grpc_dot_core__pb2.BasketResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class InstallmentTermControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Retrieve = channel.unary_unary(
                '/darsman.core.InstallmentTermController/Retrieve',
                request_serializer=core_dot_grpc_dot_core__pb2.InstallmentTermRequest.SerializeToString,
                response_deserializer=core_dot_grpc_dot_core__pb2.Response.FromString,
                )


class InstallmentTermControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Retrieve(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_InstallmentTermControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Retrieve': grpc.unary_unary_rpc_method_handler(
                    servicer.Retrieve,
                    request_deserializer=core_dot_grpc_dot_core__pb2.InstallmentTermRequest.FromString,
                    response_serializer=core_dot_grpc_dot_core__pb2.Response.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'darsman.core.InstallmentTermController', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class InstallmentTermController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Retrieve(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.core.InstallmentTermController/Retrieve',
            core_dot_grpc_dot_core__pb2.InstallmentTermRequest.SerializeToString,
            core_dot_grpc_dot_core__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class TradeControllerStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.End = channel.unary_unary(
                '/darsman.core.TradeController/End',
                request_serializer=core_dot_grpc_dot_core__pb2.EndTradeRequest.SerializeToString,
                response_deserializer=core_dot_grpc_dot_core__pb2.Response.FromString,
                )
        self.Start = channel.unary_unary(
                '/darsman.core.TradeController/Start',
                request_serializer=core_dot_grpc_dot_core__pb2.StartTradeRequest.SerializeToString,
                response_deserializer=core_dot_grpc_dot_core__pb2.Response.FromString,
                )


class TradeControllerServicer(object):
    """Missing associated documentation comment in .proto file."""

    def End(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Start(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_TradeControllerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'End': grpc.unary_unary_rpc_method_handler(
                    servicer.End,
                    request_deserializer=core_dot_grpc_dot_core__pb2.EndTradeRequest.FromString,
                    response_serializer=core_dot_grpc_dot_core__pb2.Response.SerializeToString,
            ),
            'Start': grpc.unary_unary_rpc_method_handler(
                    servicer.Start,
                    request_deserializer=core_dot_grpc_dot_core__pb2.StartTradeRequest.FromString,
                    response_serializer=core_dot_grpc_dot_core__pb2.Response.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'darsman.core.TradeController', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class TradeController(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def End(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.core.TradeController/End',
            core_dot_grpc_dot_core__pb2.EndTradeRequest.SerializeToString,
            core_dot_grpc_dot_core__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Start(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/darsman.core.TradeController/Start',
            core_dot_grpc_dot_core__pb2.StartTradeRequest.SerializeToString,
            core_dot_grpc_dot_core__pb2.Response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from .grpc.core_pb2 import (
    AddItemRequest,
    BasketApplyDiscountRequest,
    BasketAssignOwnerRequest,
    BasketRequest,
    BasketRetrieveRequest,
    EndTradeRequest,
    InstallmentTermRequest,
    RemoveItemRequest,
    StartTradeRequest,
)
from .grpc.core_pb2_grpc import (
    BasketControllerStub,
    InstallmentTermControllerStub,
    TradeControllerStub,
)

SUCCESS_CODE = "0"


def _singletonize_class_vars(cls):
    for attr_name, attr_value in cls.__dict__.items():
        if isinstance(attr_value, type):  # it's a class
            orig_cls = attr_value

            class SingletonWrapper(orig_cls):
                _instances = {}

                def __new__(cls, *args, **kwargs):
                    if cls not in cls._instances:
                        cls._instances[cls] = super().__new__(cls)
                    return cls._instances[cls]

            setattr(cls, attr_name, SingletonWrapper)
    return cls


class BasketRequestMessages:
    RETRIEVE = BasketRetrieveRequest
    CREATE = BasketRequest
    ADD_ITEM = AddItemRequest
    REMOVE_ITEM = RemoveItemRequest
    ASSIGN_OWNER = BasketAssignOwnerRequest
    APPLY_DISCOUNT = BasketApplyDiscountRequest


class BasketRPCMethodNames:
    RETRIEVE = "Retrieve"
    CREATE = "Create"
    ADD_ITEM = "AddItem"
    REMOVE_ITEM = "RemoveItem"
    ASSIGN_OWNER = "AssignOwner"
    APPLY_DISCOUNT = "ApplyDiscount"


class TradeRequestMessages:
    START = StartTradeRequest
    END = EndTradeRequest


class TradeRPCMethodNames:
    START = "Start"
    END = "End"


class InstallmentTermRequestMessages:
    RETRIEVE = InstallmentTermRequest


class InstallmentTermRPCMethodNames:
    RETRIEVE = "Retrieve"


@_singletonize_class_vars
class Stubs:
    BASKET = BasketControllerStub
    TRADE = TradeControllerStub
    INSTALLMENT_TERM = InstallmentTermControllerStub



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: core/grpc/core.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14\x63ore/grpc/core.proto\x12\x0c\x64\x61rsman.core\x1a\x1cgoogle/protobuf/struct.proto\"\xaf\x01\n\x0e\x41\x64\x64ItemRequest\x12\x17\n\x0fitem_identifier\x18\t \x01(\t\x12\x30\n\nextra_data\x18\n \x01(\x0b\x32\x17.google.protobuf.StructH\x00\x88\x01\x01\x12\x19\n\x11\x62\x61sket_identifier\x18\x0b \x01(\t\x12\x18\n\x0binstallment\x18\x0c \x01(\x08H\x01\x88\x01\x01\x42\r\n\x0b_extra_dataB\x0e\n\x0c_installment\"D\n\x1a\x42\x61sketApplyDiscountRequest\x12\x0e\n\x06\x62\x61sket\x18\x01 \x01(\t\x12\x16\n\x0e\x64iscount_token\x18\x02 \x01(\t\"D\n\x18\x42\x61sketAssignOwnerRequest\x12\x0e\n\x06\x62\x61sket\x18\x01 \x01(\t\x12\x18\n\x10owner_identifier\x18\x02 \x01(\t\"y\n\rBasketRequest\x12\x1d\n\x10owner_identifier\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x1e\n\x11\x62\x61sket_identifier\x18\x02 \x01(\tH\x01\x88\x01\x01\x42\x13\n\x11_owner_identifierB\x14\n\x12_basket_identifier\"u\n\x0e\x42\x61sketResponse\x12\x14\n\x07message\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04\x63ode\x18\x02 \x01(\tH\x01\x88\x01\x01\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.StructB\n\n\x08_messageB\x07\n\x05_code\"2\n\x15\x42\x61sketRetrieveRequest\x12\x19\n\x11\x62\x61sket_identifier\x18\x01 \x01(\t\"\x86\x01\n\x0f\x45ndTradeRequest\x12\x16\n\x0egateway_status\x18\x01 \x01(\t\x12\x1a\n\x12payment_identifier\x18\x02 \x01(\t\x12\x30\n\nextra_data\x18\x03 \x01(\x0b\x32\x17.google.protobuf.StructH\x00\x88\x01\x01\x42\r\n\x0b_extra_data\"@\n\x16InstallmentTermRequest\x12\x12\n\nidentifier\x18\x01 \x01(\t\x12\x12\n\nitem_price\x18\x02 \x01(\x05\"\x88\x01\n\x11RemoveItemRequest\x12\x17\n\x0fitem_identifier\x18\x07 \x01(\t\x12\x30\n\nextra_data\x18\x08 \x01(\x0b\x32\x17.google.protobuf.StructH\x00\x88\x01\x01\x12\x19\n\x11\x62\x61sket_identifier\x18\t \x01(\tB\r\n\x0b_extra_data\"o\n\x08Response\x12\x14\n\x07message\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x11\n\x04\x63ode\x18\x02 \x01(\tH\x01\x88\x01\x01\x12%\n\x04\x64\x61ta\x18\x03 \x03(\x0b\x32\x17.google.protobuf.StructB\n\n\x08_messageB\x07\n\x05_code\"#\n\x11StartTradeRequest\x12\x0e\n\x06\x62\x61sket\x18\x01 \x01(\t2\xdc\x03\n\x10\x42\x61sketController\x12\x41\n\x07\x41\x64\x64Item\x12\x1c.darsman.core.AddItemRequest\x1a\x16.darsman.core.Response\"\x00\x12S\n\rApplyDiscount\x12(.darsman.core.BasketApplyDiscountRequest\x1a\x16.darsman.core.Response\"\x00\x12O\n\x0b\x41ssignOwner\x12&.darsman.core.BasketAssignOwnerRequest\x1a\x16.darsman.core.Response\"\x00\x12\x45\n\x06\x43reate\x12\x1b.darsman.core.BasketRequest\x1a\x1c.darsman.core.BasketResponse\"\x00\x12G\n\nRemoveItem\x12\x1f.darsman.core.RemoveItemRequest\x1a\x16.darsman.core.Response\"\x00\x12O\n\x08Retrieve\x12#.darsman.core.BasketRetrieveRequest\x1a\x1c.darsman.core.BasketResponse\"\x00\x32g\n\x19InstallmentTermController\x12J\n\x08Retrieve\x12$.darsman.core.InstallmentTermRequest\x1a\x16.darsman.core.Response\"\x00\x32\x95\x01\n\x0fTradeController\x12>\n\x03\x45nd\x12\x1d.darsman.core.EndTradeRequest\x1a\x16.darsman.core.Response\"\x00\x12\x42\n\x05Start\x12\x1f.darsman.core.StartTradeRequest\x1a\x16.darsman.core.Response\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'core.grpc.core_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_ADDITEMREQUEST']._serialized_start=69
  _globals['_ADDITEMREQUEST']._serialized_end=244
  _globals['_BASKETAPPLYDISCOUNTREQUEST']._serialized_start=246
  _globals['_BASKETAPPLYDISCOUNTREQUEST']._serialized_end=314
  _globals['_BASKETASSIGNOWNERREQUEST']._serialized_start=316
  _globals['_BASKETASSIGNOWNERREQUEST']._serialized_end=384
  _globals['_BASKETREQUEST']._serialized_start=386
  _globals['_BASKETREQUEST']._serialized_end=507
  _globals['_BASKETRESPONSE']._serialized_start=509
  _globals['_BASKETRESPONSE']._serialized_end=626
  _globals['_BASKETRETRIEVEREQUEST']._serialized_start=628
  _globals['_BASKETRETRIEVEREQUEST']._serialized_end=678
  _globals['_ENDTRADEREQUEST']._serialized_start=681
  _globals['_ENDTRADEREQUEST']._serialized_end=815
  _globals['_INSTALLMENTTERMREQUEST']._serialized_start=817
  _globals['_INSTALLMENTTERMREQUEST']._serialized_end=881
  _globals['_REMOVEITEMREQUEST']._serialized_start=884
  _globals['_REMOVEITEMREQUEST']._serialized_end=1020
  _globals['_RESPONSE']._serialized_start=1022
  _globals['_RESPONSE']._serialized_end=1133
  _globals['_STARTTRADEREQUEST']._serialized_start=1135
  _globals['_STARTTRADEREQUEST']._serialized_end=1170
  _globals['_BASKETCONTROLLER']._serialized_start=1173
  _globals['_BASKETCONTROLLER']._serialized_end=1649
  _globals['_INSTALLMENTTERMCONTROLLER']._serialized_start=1651
  _globals['_INSTALLMENTTERMCONTROLLER']._serialized_end=1754
  _globals['_TRADECONTROLLER']._serialized_start=1757
  _globals['_TRADECONTROLLER']._serialized_end=1906
# @@protoc_insertion_point(module_scope)



from typing import ClassVar, Optional, Type, TypeVar
from uuid import UUID

from google.protobuf.json_format import MessageToDict
from google.protobuf.message import Message
from pydantic import BaseModel, validate_call

from . import config
from .api_constants import (
    SUCCESS_CODE,
    BasketRequestMessages,
    BasketRPCMethodNames,
    InstallmentTermRequestMessages,
    InstallmentTermRPCMethodNames,
    Stubs,
    TradeRequestMessages,
    TradeRPCMethodNames,
)
from .schemas import InstallmentTerm as InstallmentTermSchema

M = TypeVar("M", bound=BaseModel)


class ValidationError(Exception):
    pass


class ResponseCodeError(Exception):
    pass


class RPCClient(BaseModel):
    _instances: ClassVar[dict] = {}
    stub_cls: ClassVar

    def __new__(cls, *args, **kwargs):
        # Ensure that each subclass has its own singleton instance
        if cls not in cls._instances:
            cls._instances[cls] = super().__new__(cls)
        return cls._instances[cls]

    @validate_call(config=dict(arbitrary_types_allowed=True))
    async def perform_rpc(
        self,
        *,
        rpc_method_name: str,
        request_message: Optional[Message] = None,
        response_model: Optional[Type[M]] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ) -> list[M] | dict:
        """Perform the remote-precedure-call.

        Parameters:
            rpc_method_name: The name of the RPC method (e.g., `Retrieve`).
            request_message: The request message instance.
            timeout: RPC timeout.
            wait_for_ready: Wait for Channel to be ready in case is not.
        """
        if config.CHANNEL is None:
            await config.construct_channel()

        RPCMethod = getattr(self.stub_cls(config.CHANNEL), rpc_method_name, None)
        if not RPCMethod:
            raise ValidationError("Invalid RPC method name.")

        metadata = [("api-key", config.grpc_settings.API_KEY)]
        if request_id:
            metadata.append(("x-request-id", str(request_id)))

        response = await RPCMethod(
            request_message,
            metadata=metadata,
            timeout=config.grpc_settings.TIMEOUT if timeout is None else timeout,
            wait_for_ready=config.grpc_settings.WAIT_FOR_READY
            if wait_for_ready is None
            else wait_for_ready,
        )

        # Raise error if response is not successful
        if response.code != SUCCESS_CODE:
            raise ResponseCodeError(response.code)

        response_dict = MessageToDict(response, preserving_proto_field_name=True)
        data = response_dict.get("data", [])

        if not response_model:
            return data

        return [response_model.model_validate(item) for item in data]


class Basket(RPCClient):
    stub_cls = Stubs.BASKET

    @validate_call
    async def create(
        self,
        owner_identifier: Optional[str] = None,
        basket_identifier: Optional[str] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Create a new basket instance."""
        # Assign attributes to the request message
        request = BasketRequestMessages.CREATE()
        if owner_identifier:
            request.owner_identifier = owner_identifier
        if basket_identifier:
            request.basket_identifier = basket_identifier

        return await self.perform_rpc(
            rpc_method_name=BasketRPCMethodNames.CREATE,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def retrieve(
        self,
        basket_identifier: str,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Retrieve a basket instance."""
        # Assign attributes to the request message
        request = BasketRequestMessages.RETRIEVE(basket_identifier=basket_identifier)

        return await self.perform_rpc(
            rpc_method_name=BasketRPCMethodNames.RETRIEVE,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def add_item(
        self,
        item_identifier: str,
        basket_identifier: str,
        installment: Optional[bool] = False,
        extra_data: Optional[dict] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Add item to an existing basket."""
        # Set the keyword attribute in the request message
        request = BasketRequestMessages.ADD_ITEM(
            item_identifier=item_identifier,
            basket_identifier=basket_identifier,
            installment=installment,
        )
        if extra_data:
            request.extra_data.update(extra_data)

        return await self.perform_rpc(
            rpc_method_name=BasketRPCMethodNames.ADD_ITEM,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def remove_item(
        self,
        item_identifier: str,
        basket_identifier: str,
        extra_data: Optional[dict] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Remove an item from an existing basket."""
        # Set the keyword attribute in the request message
        request = BasketRequestMessages.REMOVE_ITEM(
            item_identifier=item_identifier, basket_identifier=basket_identifier
        )
        if extra_data:
            request.extra_data.update(extra_data)

        return await self.perform_rpc(
            rpc_method_name=BasketRPCMethodNames.REMOVE_ITEM,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def assign_owner(
        self,
        owner_identifier: str,
        basket_identifier: str,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Assign the specified owner to the specified basket."""
        # Set the keyword attribute in the request message
        request = BasketRequestMessages.ASSIGN_OWNER(
            owner_identifier=owner_identifier, basket=basket_identifier
        )

        return await self.perform_rpc(
            rpc_method_name=BasketRPCMethodNames.ASSIGN_OWNER,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def apply_discount(
        self,
        basket_identifier: str,
        discount_token: str,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Assign the specified owner to the specified basket."""
        # Set the keyword attribute in the request message
        request = BasketRequestMessages.APPLY_DISCOUNT(
            discount_token=discount_token, basket=basket_identifier
        )

        return await self.perform_rpc(
            rpc_method_name=BasketRPCMethodNames.APPLY_DISCOUNT,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )


class Trade(RPCClient):
    stub_cls = Stubs.TRADE

    @validate_call
    async def start(
        self,
        basket_identifier: str,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Start the trade process (paying a basket)."""

        # Set the keyword attribute in the START request message
        request = TradeRequestMessages.START(basket=basket_identifier)
        return await self.perform_rpc(
            rpc_method_name=TradeRPCMethodNames.START,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )

    @validate_call
    async def end(
        self,
        gateway_status: str,
        payment_identifier: str,
        extra_data: Optional[dict] = None,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """End the trade process (verify payment)."""

        # Set the keyword attribute in the END request message
        request = TradeRequestMessages.END(
            gateway_status=gateway_status, payment_identifier=payment_identifier
        )
        if extra_data:
            request.extra_data.update(extra_data)
        return await self.perform_rpc(
            rpc_method_name=TradeRPCMethodNames.END,
            request_message=request,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )


class InstallmentTerm(RPCClient):
    stub_cls = Stubs.INSTALLMENT_TERM

    @validate_call
    async def retrieve(
        self,
        identifier: str | UUID,
        item_price: int,
        request_id: Optional[str | int | UUID] = None,
        timeout: Optional[float] = None,
        wait_for_ready: Optional[bool] = None,
    ):
        """Retrieve installment term for an item with `item_price`."""
        # Assign attributes to the request message
        request = InstallmentTermRequestMessages.RETRIEVE(
            identifier=str(identifier), item_price=item_price
        )

        return await self.perform_rpc(
            rpc_method_name=InstallmentTermRPCMethodNames.RETRIEVE,
            request_message=request,
            response_model=InstallmentTermSchema,
            request_id=request_id,
            timeout=timeout,
            wait_for_ready=wait_for_ready,
        )
